@@ -1,12 +1,15 @@
-import requests
+"""Support for cloud and local Smappee API."""
 import datetime as dt
 import functools
+import numbers
+import pytz
+import requests
+from cachetools import TTLCache
+from requests.exceptions import HTTPError, ConnectTimeout, ReadTimeout, \
+    ConnectionError as RequestsConnectionError
+from requests_oauthlib import OAuth2Session
 from .config import config
 from .helper import urljoin
-from requests.exceptions import HTTPError, ConnectTimeout, ReadTimeout, ConnectionError
-from requests_oauthlib import OAuth2Session
-import pytz
-import numbers
 
 
 def authenticated(func):
@@ -23,7 +26,8 @@ def authenticated(func):
     return wrapper
 
 
-class SmappeeApi(object):
+class SmappeeApi:
+    """Public Smappee cloud API wrapper."""
 
     def __init__(
             self,
@@ -66,14 +70,22 @@ class SmappeeApi(object):
 
     @authenticated
     def get_metering_configuration(self, service_location_id):
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "meteringconfiguration")
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "meteringconfiguration"
+        )
         r = requests.get(url, headers=self.headers)
         r.raise_for_status()
         return r.json()
 
     @authenticated
     def get_service_location_info(self, service_location_id):
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "info")
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "info"
+        )
         r = requests.get(url, headers=self.headers)
         r.raise_for_status()
         return r.json()
@@ -91,7 +103,11 @@ class SmappeeApi(object):
             7 = ...
             8 = ...
         """
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "consumption")
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "consumption"
+        )
         d = self._get_consumption(url=url, start=start, end=end, aggregation=aggregation)
         for block in d['consumptions']:
             if 'alwaysOn' not in block.keys():
@@ -101,12 +117,24 @@ class SmappeeApi(object):
 
     @authenticated
     def get_sensor_consumption(self, service_location_id, sensor_id, start, end, aggregation):
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "sensor", sensor_id, "consumption")
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "sensor",
+            sensor_id,
+            "consumption"
+        )
         return self._get_consumption(url=url, start=start, end=end, aggregation=aggregation)
 
     @authenticated
     def get_switch_consumption(self, service_location_id, switch_id, start, end, aggregation):
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "switch", switch_id, "consumption")
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "switch",
+            switch_id,
+            "consumption"
+        )
         return self._get_consumption(url=url, start=start, end=end, aggregation=aggregation)
 
     def _get_consumption(self, url, start, end, aggregation):
@@ -125,7 +153,11 @@ class SmappeeApi(object):
     def get_events(self, service_location_id, appliance_id, start, end, max_number=None):
         start, end = self._to_milliseconds(start), self._to_milliseconds(end)
 
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "events")
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "events"
+        )
         params = {
             "from": start,
             "to": end,
@@ -138,14 +170,26 @@ class SmappeeApi(object):
 
     @authenticated
     def get_actuator_state(self, service_location_id, actuator_id):
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "actuator", actuator_id, "state")
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "actuator",
+            actuator_id,
+            "state"
+        )
         r = requests.get(url, headers=self.headers)
         r.raise_for_status()
         return r.text
 
     @authenticated
     def set_actuator_state(self, service_location_id, actuator_id, state_id, duration=None):
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "actuator", actuator_id, state_id)
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "actuator",
+            actuator_id,
+            state_id
+        )
         data = {} if duration is None else {"duration": duration}
         r = requests.post(url, headers=self.headers, json=data)
         r.raise_for_status()
@@ -153,7 +197,13 @@ class SmappeeApi(object):
 
     @authenticated
     def get_actuator_connection_state(self, service_location_id, actuator_id):
-        url = urljoin(config['API_URL'][self._farm]['servicelocation_url'], service_location_id, "actuator", actuator_id, "connectionstate")
+        url = urljoin(
+            config['API_URL'][self._farm]['servicelocation_url'],
+            service_location_id,
+            "actuator",
+            actuator_id,
+            "connectionstate"
+        )
         r = requests.get(url, headers=self.headers)
         r.raise_for_status()
         return r.text
@@ -189,7 +239,8 @@ class SmappeeApi(object):
         return token
 
 
-class SmappeeLocalApi(object):
+class SmappeeLocalApi:
+    """Smappee local API wrapper."""
 
     def __init__(
             self,
@@ -198,6 +249,13 @@ class SmappeeLocalApi(object):
         self._ip = ip
         self.session = requests.Session()
 
+        # default indices for Smappee Energy and Solar
+        self.consumption_indices = ['phase0ActivePower', 'phase1ActivePower', 'phase2ActivePower']
+        self.production_indices = ['phase3ActivePower', 'phase4ActivePower', 'phase5ActivePower']
+
+        # cache instantaneous load
+        self.load_cache = TTLCache(maxsize=1, ttl=5)
+
     @property
     def host(self):
         return f'http://{self._ip}/gateway/apipublic'
@@ -212,16 +270,17 @@ class SmappeeLocalApi(object):
             r = self.session.post(_url,
                                   data=data,
                                   headers=self.headers,
-                                  timeout=5)
+                                  timeout=2)
             r.raise_for_status()
 
             msg = r.json()
-            if not retry and 'error' in msg and msg['error'] == 'Error not authenticated. Use Logon first!':
+            if not retry and 'error' in msg \
+                    and msg['error'] == 'Error not authenticated. Use Logon first!':
                 self.logon()
                 return self._post(url=url, data=data, retry=True)
 
             return msg
-        except (ConnectTimeout, ReadTimeout, ConnectionError, HTTPError):
+        except (ConnectTimeout, ReadTimeout, RequestsConnectionError, HTTPError):
             return None
 
     def logon(self):
@@ -230,6 +289,22 @@ class SmappeeLocalApi(object):
     def load_advanced_config(self):
         return self._post(url='advancedConfigPublic', data='load')
 
+    def load_channels_config(self):
+        # Method only available on Smappee2-series devices
+
+        # reset consumption and production indices
+        self.consumption_indices, self.production_indices = [], []
+
+        cc = self._post(url='channelsConfigPublic', data='load')
+        for input_channel in cc['inputChannels']:
+            if input_channel['inputChannelConnection'] == 'GRID':
+                if input_channel['inputChannelType'] == 'CONSUMPTION':
+                    self.consumption_indices.append(f'phase{input_channel["ctInput"]}ActivePower')
+                elif input_channel['inputChannelType'] == 'PRODUCTION':
+                    self.production_indices.append(f'phase{input_channel["ctInput"]}ActivePower')
+
+        return cc
+
     def load_config(self):
         return self._post(url='configPublic', data='load')
 
@@ -240,18 +315,30 @@ class SmappeeLocalApi(object):
         return self._post(url='instantaneous', data='loadInstantaneous')
 
     def active_power(self, solar=False):
+        """
+        Get the current active power consumption or solar production. Result is cached.
+
+        :param solar:
+        :return:
+        """
+        if 'instantaneous' in self.load_cache:
+            return self.load_cache['instantaneous']
+
         inst = self.load_instantaneous()
 
         if inst is None:
             return None
 
-        if not solar:
-            power_keys = ['phase0ActivePower', 'phase1ActivePower', 'phase2ActivePower']
-        else:
-            power_keys = ['phase3ActivePower', 'phase4ActivePower', 'phase5ActivePower']
+        power_keys = self.production_indices if solar else self.consumption_indices
 
         values = [float(i['value']) for i in inst if i['key'] in power_keys]
-        return int(sum(values) / 1000)
+        if values:
+            power = int(sum(values) / 1000)
+        else:
+            power = 0
+        self.load_cache['instantaneous'] = power
+
+        return power
 
     def set_actuator_state(self, service_location_id, actuator_id, state_id, duration=None):
         if state_id == 'ON_ON':
